@@ -32,14 +32,17 @@ from schooltool.common import SchoolToolMessage as _
 class ICourse(Interface):
     """Courses are similar to groups, membership is restricted to Sections."""
 
-    title = zope.schema.TextLine(
-        title=_("Title"),
-        description=_("Title of the course."))
+    __name__ = zope.schema.TextLine(
+        title=_("SchoolTool ID"),
+        description=_(
+            """An internal identifier of this course."""),
+        required=True)
+
+    title = zope.schema.TextLine(title=_("Title"))
 
     description = zope.schema.Text(
         title=_("Description"),
-        required=False,
-        description=_("Description of the course."))
+        required=False)
 
     sections = Attribute(
         """The Sections that implement this course material,
@@ -47,19 +50,18 @@ class ICourse(Interface):
 
     course_id = zope.schema.TextLine(
         title=_("Course ID"),
-        required=False,
-        description=_("Localy used course identifier."))
+        description=_(
+            """School identifier of this course."""),
+        required=False)
 
     government_id = zope.schema.TextLine(
-        title=_("Government ID"),
+        title=_("Alternate ID"),
         required=False,
-        description=_("Course identifier used by the government."))
+        description=_("Additional identifier for outside tracking or other purposes."))
 
     credits = zope.schema.Decimal(
         title=_("Credits"),
-        required=False,
-        description=_("Amount of credits for this course."))
-
+        required=False)
 
 
 class ICourseContainer(IContainer):
@@ -80,7 +82,7 @@ class ICourseContained(ICourse, IContained):
     containers(ICourseContainer)
 
 
-class ISection(IGroup):
+class ISectionBase(IGroup):
     """Sections are groups of users in a particular meeting of a Course."""
 
     label = zope.schema.TextLine(
@@ -92,13 +94,11 @@ class ISection(IGroup):
 
     title = zope.schema.TextLine(
         title=_("Title"),
-        required=True,
-        description=_("Title for the section."))
+        required=True)
 
     description = zope.schema.Text(
         title=_("Description"),
-        required=False,
-        description=_("Description of the section."))
+        required=False)
 
     instructors = Attribute(
         """A list of Person objects in the role of instructor""")
@@ -122,6 +122,15 @@ class ISection(IGroup):
         """Chain of sections linked by previous/next with this one.""")
 
 
+class ISection(ISectionBase):
+
+    __name__ = zope.schema.TextLine(
+        title=_("SchoolTool ID"),
+        description=_(
+            """An internal identifier of this section."""),
+        required=True)
+
+
 class ISectionContainer(IContainer):
     """A container for Sections."""
 
