@@ -21,6 +21,8 @@ group views.
 
 $Id$
 """
+import zc.table.table
+import zc.table.column
 from zope.app.dependable.interfaces import IDependable
 from zope.cachedescriptors.property import Lazy
 from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
@@ -42,43 +44,35 @@ from zope.traversing.browser.absoluteurl import absoluteURL
 from zope.i18n import translate
 from z3c.form import field, button, form
 from z3c.form.interfaces import HIDDEN_MODE
-from zc.table import table, column
 from zc.table.interfaces import ISortableColumn
 
-from schooltool.common import SchoolToolMessage as _
 from schooltool.app.interfaces import ISchoolToolApplication
 from schooltool.skin.containers import TableContainerView
 from schooltool.app.browser.app import BaseAddView, BaseEditView
 from schooltool.app.browser.app import ContentTitle
+from schooltool.app.browser.app import EditRelationships
+from schooltool.app.browser.app import RelationshipAddTableMixin
+from schooltool.app.browser.app import RelationshipRemoveTableMixin
+from schooltool.app.browser.app import RelationshipViewBase
 from schooltool.person.interfaces import IPerson
+from schooltool.basicperson.browser.person import BasicPersonTable
+from schooltool.basicperson.browser.person import EditPersonRelationships
 from schooltool.course.interfaces import ISection
-from schooltool.table.interfaces import ITableFormatter
 from schooltool.schoolyear.interfaces import ISchoolYear
 from schooltool.schoolyear.interfaces import ISchoolYearContainer
 from schooltool.group.group import Group
 from schooltool.group.interfaces import IGroup
 from schooltool.group.interfaces import IGroupMember
 from schooltool.group.interfaces import IGroupContainer, IGroupContained
-from schooltool.app.browser.app import RelationshipViewBase
-from schooltool.app.browser.app import FlourishRelationshipViewBase
-from schooltool.table.table import FilterWidget
-from schooltool.table.table import SchoolToolTableFormatter
-from schooltool.table.table import LocaleAwareGetterColumn
 from schooltool.skin.flourish.viewlet import Viewlet
 from schooltool.common.inlinept import InheritTemplate
 from schooltool.common.inlinept import InlineViewPageTemplate
-from schooltool.skin.flourish.containers import TableContainerView as FlourishTableContainerView
-from schooltool.skin.flourish.containers import ContainerDeleteView
-from schooltool.skin.flourish.page import RefineLinksViewlet
-from schooltool.skin.flourish.page import LinkViewlet
-from schooltool.skin.flourish.page import Page
-from schooltool.skin.flourish.page import ModalFormLinkViewlet
-from schooltool.skin.flourish.page import Content
-from schooltool.skin.flourish.form import Form
-from schooltool.skin.flourish.form import AddForm
-from schooltool.skin.flourish.form import DialogForm
-from schooltool.skin.flourish.form import DisplayForm
-from schooltool.skin.flourish.page import TertiaryNavigationManager
+from schooltool.skin import flourish
+from schooltool import table
+
+from schooltool.common import SchoolToolMessage as _
+from schooltool.basicperson.browser.person import FlourishPersonIDCardsViewBase
+from schooltool.report.browser.report import RequestReportDownloadDialog
 
 
 class GroupContainerAbsoluteURLAdapter(BrowserView):
@@ -136,8 +130,9 @@ class GroupView(BrowserView):
 
     def renderPersonTable(self):
         persons = ISchoolToolApplication(None)['persons']
-        formatter = getMultiAdapter((persons, self.request), ITableFormatter)
-        formatter.setUp(table_formatter=table.StandaloneFullFormatter,
+        formatter = getMultiAdapter((persons, self.request),
+                                    table.interfaces.ITableFormatter)
+        formatter.setUp(table_formatter=zc.table.table.StandaloneFullFormatter,
                         items=self.getPersons(),
                         batch_size=0)
         return formatter.render()
@@ -247,12 +242,12 @@ class FlourishGroupsViewlet(Viewlet):
         return canAccess(self.context.__parent__, '__delitem__')
 
 
-class FlourishGroupFilterWidget(FilterWidget):
+class FlourishGroupFilterWidget(table.table.FilterWidget):
 
     template = ViewPageTemplateFile('templates/f_group_filter.pt')
 
 
-class SchoolYearColumn(column.GetterColumn):
+class SchoolYearColumn(zc.table.column.GetterColumn):
 
     def getter(self, item, formatter):
         schoolyear = ISchoolYear(item.__parent__)
@@ -263,10 +258,10 @@ class SchoolYearColumn(column.GetterColumn):
         return schoolyear.first
 
 
-class FlourishGroupTableFormatter(SchoolToolTableFormatter):
+class FlourishGroupTableFormatter(table.table.SchoolToolTableFormatter):
 
     def columns(self):
-        title = LocaleAwareGetterColumn(
+        title = table.column.LocaleAwareGetterColumn(
             name='title',
             title=_(u"Title"),
             getter=lambda i, f: i.title,
@@ -285,7 +280,7 @@ class FlourishGroupTableFormatter(SchoolToolTableFormatter):
         return formatter()
 
 
-class FlourishGroupListView(FlourishRelationshipViewBase):
+class FlourishGroupListView(EditRelationships):
 
     current_title = _('Current groups')
     available_title = _('Available groups')
@@ -350,7 +345,7 @@ class FlourishGroupListView(FlourishRelationshipViewBase):
         return "%s.%s" % (schoolyear.__name__, item.__name__)
 
 
-class GroupsTertiaryNavigationManager(TertiaryNavigationManager):
+class GroupsTertiaryNavigationManager(flourish.page.TertiaryNavigationManager):
 
     template = InlineViewPageTemplate("""
         <ul tal:attributes="class view/list_class">
@@ -381,15 +376,15 @@ class GroupsTertiaryNavigationManager(TertiaryNavigationManager):
         return result
 
 
-class GroupsAddLinks(RefineLinksViewlet):
+class GroupsAddLinks(flourish.page.RefineLinksViewlet):
     """Manager for Add links in GroupsView"""
 
 
-class GroupImportLinks(RefineLinksViewlet):
+class GroupImportLinks(flourish.page.RefineLinksViewlet):
     """Manager for group import links."""
 
 
-class GroupLinks(RefineLinksViewlet):
+class GroupLinks(flourish.page.RefineLinksViewlet):
     """Manager for public links in GroupView"""
 
     @property
@@ -397,7 +392,7 @@ class GroupLinks(RefineLinksViewlet):
         return self.context.title
 
 
-class GroupAddLinks(RefineLinksViewlet):
+class GroupAddLinks(flourish.page.RefineLinksViewlet):
     """Manager for Add links in GroupView"""
 
     def render(self):
@@ -407,7 +402,7 @@ class GroupAddLinks(RefineLinksViewlet):
             return super(GroupAddLinks, self).render()
 
 
-class GroupManageActionsLinks(RefineLinksViewlet):
+class GroupManageActionsLinks(flourish.page.RefineLinksViewlet):
     """Manager for Action links in GroupView"""
 
     body_template = InlineViewPageTemplate("""
@@ -441,7 +436,7 @@ class GroupManageActionsLinks(RefineLinksViewlet):
                 return super(GroupManageActionsLinks, self).render()
 
 
-class GroupDeleteLink(ModalFormLinkViewlet):
+class GroupDeleteLink(flourish.page.ModalFormLinkViewlet):
 
     @property
     def dialog_title(self):
@@ -468,7 +463,7 @@ class GroupsActiveTabMixin(object):
         return result
 
 
-class GroupAddLinkViewlet(LinkViewlet, GroupsActiveTabMixin):
+class GroupAddLinkViewlet(flourish.page.LinkViewlet, GroupsActiveTabMixin):
 
     @property
     def url(self):
@@ -500,10 +495,12 @@ class GroupContainerTitle(ContentTitle):
                  mapping={'schoolyear': schoolyear.title})
 
 
-class FlourishGroupsView(FlourishTableContainerView,
+class FlourishGroupsView(flourish.page.Page,
                          GroupsActiveTabMixin):
 
-    content_template = ViewPageTemplateFile('templates/f_groups.pt')
+    content_template = InlineViewPageTemplate('''
+      <div tal:content="structure context/schooltool:content/ajax/view/container/table" />
+    ''')
 
     @property
     def title(self):
@@ -511,21 +508,77 @@ class FlourishGroupsView(FlourishTableContainerView,
         return _('Groups for ${schoolyear}',
                  mapping={'schoolyear': schoolyear.title})
 
-    @property
+    @Lazy
     def container(self):
         schoolyear = self.schoolyear
         return IGroupContainer(schoolyear)
 
-    def getColumnsAfter(self):
-        description = column.GetterColumn(
+
+class GroupsTable(table.ajax.Table):
+
+    def columns(self):
+        default = table.ajax.Table.columns(self)
+        description = zc.table.column.GetterColumn(
             name='description',
             title=_('Description'),
             getter=lambda i, f: i.description or '',
             )
-        return [description]
+        return default + [description]
 
 
-class FlourishGroupContainerDeleteView(ContainerDeleteView):
+class GroupsTableFilter(table.ajax.TableFilter):
+
+    title = _("Group title")
+
+
+class GroupsTableSchoolYear(flourish.viewlet.Viewlet):
+
+    template = InlineViewPageTemplate('''
+      <input type="hidden" name="schoolyear_id"
+             tal:define="schoolyear_id view/view/schoolyear/__name__|nothing"
+             tal:condition="schoolyear_id"
+             tal:attributes="value schoolyear_id" />
+    ''')
+
+
+class GroupsWithSYTable(GroupsTable):
+
+    def columns(self):
+        default = table.ajax.Table.columns(self)
+        schoolyear = SchoolYearColumn(
+            name='schoolyear',
+            title=_(u'School Year'),
+            subsort=True)
+        directlyProvides(schoolyear, ISortableColumn)
+        return default + [schoolyear]
+
+    def sortOn(self):
+        return (('schoolyear', True), ("title", False))
+
+
+class GroupListAddRelationshipTable(RelationshipAddTableMixin,
+                                    GroupsWithSYTable):
+
+    def updateFormatter(self):
+        ommit = self.view.getOmmitedItems()
+        available = self.view.getAvailableItems()
+        columns = self.columns()
+        self.setUp(formatters=[table.table.url_cell_formatter],
+                   columns=columns,
+                   ommit=ommit,
+                   items=available,
+                   table_formatter=self.table_formatter,
+                   batch_size=self.batch_size,
+                   prefix=self.__name__,
+                   css_classes={'table': 'data relationships-table'})
+
+
+class GroupListRemoveRelationshipTable(RelationshipRemoveTableMixin,
+                                       GroupsWithSYTable):
+    pass
+
+
+class FlourishGroupContainerDeleteView(flourish.containers.ContainerDeleteView):
 
     def nextURL(self):
         if 'CONFIRM' in self.request:
@@ -535,12 +588,12 @@ class FlourishGroupContainerDeleteView(ContainerDeleteView):
                 'groups',
                 schoolyear.__name__)
             return url
-        return ContainerDeleteView.nextURL(self)
+        return flourish.containers.ContainerDeleteView.nextURL(self)
 
 
-class FlourishGroupView(DisplayForm):
+class FlourishGroupView(flourish.form.DisplayForm):
 
-    template = InheritTemplate(Page.template)
+    template = InheritTemplate(flourish.page.Page.template)
     content_template = ViewPageTemplateFile('templates/f_group_view.pt')
     fields = field.Fields(IGroup)
     fields = fields.select('title', 'description')
@@ -573,34 +626,16 @@ class FlourishGroupView(DisplayForm):
             if not widget.value:
                 widget.mode = HIDDEN_MODE
 
-    # XXX: GroupView.getPersons uses canAccess on the member title
-    #      should we do the same here?
-    @property
-    def members_table(self):
-        return self.getTable(list(self.context.members), 'members')
-
     def has_members(self):
         return bool(list(self.context.members))
 
-    @property
-    def leaders_table(self):
-        return self.getTable(list(self.context.leaders), 'leaders',
-                             filter=lambda l: l)
-
-    def getTable(self, items, prefix, **kw):
-        persons = ISchoolToolApplication(None)['persons']
-        result = getMultiAdapter((persons, self.request), ITableFormatter)
-        result.setUp(table_formatter=table.StandaloneFullFormatter, items=items,
-                     prefix=prefix, **kw)
-        return result
-
     def has_leaders(self):
         return bool(list(self.context.leaders))
 
 
-class FlourishGroupAddView(AddForm):
+class FlourishGroupAddView(flourish.form.AddForm):
 
-    template = InheritTemplate(Page.template)
+    template = InheritTemplate(flourish.page.Page.template)
     label = None
     legend = _('Group Information')
     fields = field.Fields(IGroup)
@@ -611,7 +646,7 @@ class FlourishGroupAddView(AddForm):
         self.actions['add'].addClass('button-ok')
         self.actions['cancel'].addClass('button-cancel')
 
-    @button.buttonAndHandler(_('Add'))
+    @button.buttonAndHandler(_('Submit'), name='add')
     def handleAdd(self, action):
         super(FlourishGroupAddView, self).handleAdd.func(self, action)
 
@@ -650,9 +685,9 @@ class FlourishGroupAddView(AddForm):
                  mapping={'schoolyear': schoolyear.title})
 
 
-class FlourishGroupEditView(Form, form.EditForm):
+class FlourishGroupEditView(flourish.form.Form, form.EditForm):
 
-    template = InheritTemplate(Page.template)
+    template = InheritTemplate(flourish.page.Page.template)
     label = None
     legend = _('Group Information')
     fields = field.Fields(IGroup)
@@ -685,7 +720,7 @@ class FlourishGroupEditView(Form, form.EditForm):
         self.actions['cancel'].addClass('button-cancel')
 
 
-class FlourishGroupDeleteView(DialogForm, form.EditForm):
+class FlourishGroupDeleteView(flourish.form.DialogForm, form.EditForm):
     """View used for confirming deletion of a group."""
 
     dialog_submit_actions = ('apply',)
@@ -711,7 +746,23 @@ class FlourishGroupDeleteView(DialogForm, form.EditForm):
         self.actions['cancel'].addClass('button-cancel')
 
 
-class FlourishMemberViewPersons(FlourishRelationshipViewBase):
+class GroupMembersTable(BasicPersonTable):
+
+    prefix = "members"
+
+    def items(self):
+        return self.indexItems(self.context.members)
+
+
+class GroupLeadersTable(BasicPersonTable):
+
+    prefix = "leaders"
+
+    def items(self):
+        return self.indexItems(self.context.leaders)
+
+
+class FlourishMemberViewPersons(EditPersonRelationships):
     """View class for adding / removing members to / from a group."""
 
     @property
@@ -721,28 +772,15 @@ class FlourishMemberViewPersons(FlourishRelationshipViewBase):
     current_title = _("Current Members")
     available_title = _("Add Members")
 
-    def setUpTables(self):
-        self.available_table = self.createTableFormatter(
-            ommit=self.getOmmitedItems(),
-            prefix="add_item")
-
-        self.selected_table = self.createTableFormatter(
-            filter=lambda l: l,
-            items=self.getSelectedItems(),
-            prefix="remove_item")
-
     def getSelectedItems(self):
         """Return a list of current group memebers."""
         return filter(IPerson.providedBy, self.context.members)
 
-    def getAvailableItemsContainer(self):
-        return ISchoolToolApplication(None)['persons']
-
     def getCollection(self):
         return self.context.members
 
 
-class FlourishManageGroupsOverview(Content):
+class FlourishManageGroupsOverview(flourish.page.Content):
 
     body_template = ViewPageTemplateFile(
         'templates/f_manage_groups_overview.pt')
@@ -763,3 +801,22 @@ class FlourishManageGroupsOverview(Content):
     @property
     def groups(self):
         return IGroupContainer(self.schoolyear, None)
+
+
+class FlourishRequestGroupIDCardsView(RequestReportDownloadDialog):
+
+    def nextURL(self):
+        return absoluteURL(self.context, self.request) + '/group_id_cards.pdf'
+
+
+class FlourishGroupIDCardsView(FlourishPersonIDCardsViewBase):
+
+    @property
+    def title(self):
+        return _('ID Cards for Group: ${group}',
+                 mapping={'group': self.context.title})
+
+    def persons(self):
+        result = [self.getPersonData(person)
+                  for person in self.context.members]
+        return result
