@@ -125,6 +125,8 @@ class IndexedTableFormatter(SchoolToolTableFormatter):
         return items
 
     def ommit(self, items, ommited_items):
+        if not ommited_items:
+            return items
         ommited_items = self.indexItems(ommited_items)
         ommited_ids = set([item['id'] for item in ommited_items])
         return [item for item in items
